;
; channel15_2.asm - channel 15 command execution
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

channel15_cont	.proc

		ldx #commands2-commands
htnuo		dex
		ldy commands,x
		beq cmdnotimp
		cmp commands,x
		bne htnuo
                .if SUPERCPU=1
                txa
                asl
                tax
                .else
		lda commandshi-1,x
		pha
		lda commandslo-1,x
		pha
                .fi
                #ldy_l_a path_exp+2
		lda path_exp+1
                cmp #"-"
                .if SUPERCPU=1
                jmp (commandslo-2,x)
                .else
		rts
                .fi

cmdnotimp	.proc
		sec
		gcs LWOP_reporterror_31_xit;Unknown command
		.pend
;-------------------------------
;W-0 write protect off
;W-1 write protect on
;
cmd_w		.proc
		bne cmdnotimp
                jsr OP_get_current_drive_type
                tya
                eor #$31
                lsr
                bne cmdnotimp
		rol
                sta drive_allowwrite_exp,x
ok              rts			;-=> exit, OK
;-------------------------------
;U0>P0 Power off
;U0>P1 Power on
;
cutil_p         .proc                   ;"U0>P"+CHR$(<powermode>)
                jsr p4_null_one
		ora #standby_now1_d
                tax
		eor #standby_now2_d^standby_now1_d

try2            .proc
		pha
                #phx_a
                jsr select_drive
                pla
		jsr ata_command
                pla
		bcc ok
		jsr ata_command
ok		jmp LWOP_reporterror_xit
		.pend
;-------------------------------
;U0>E0 Load disk
;U0>E1 Eject disk
;
cutil_e         .proc
		jsr cucc
                eor #3
                ldx #ATAPI_CMD_START_STOP
                gne cutil_l.in

cucc            jsr XIT_init_drive2
		jsr OP_atapi_packet

p4_null_one	.proc
		lda path_exp+4
                eor #$30
		lsr
cmdnotimpx      bne cmdnotimp
                rol
		rts
                .pend
;-------------------------------
;U User commands
;
cmd_user	.proc
		cmp #"0"
		beq usercmd0
		and #$0f
		cmp #10
		beq usercmdreset
		cmp #9
		bne cmdnotimp
;-------------------------------
;UI/U9 Identify
;
usercmd9	.proc
                cpy #"-"
                beq cmd_w.ok		;ui-
                cpy #"+"
                beq cmd_w.ok		;ui+
                lda #$80+73
		gne LWOP_err
		.pend
;-------------------------------
;UJ/U: Reset
;
usercmdreset	.proc
		lda current_drive_exp
		jmp OP_XIT_hdinit_2	;reconfigure drive
		.pend
;-------------------------------
;U0> Change utility
;
usercmd0	.proc
		tya
                and #$1f
		cmp #$1e
		bne cmd_w.ok		;ignore burst commands and u0
		lda path_exp+3
in		cmp #4
		bcc nnumber
		cmp #31			;"U0>"+CHR$(<drivenumber>)
		bcs nnumber
		#ldx_l_ current_drive_exp
		sta current_drivenumbers_exp,x
                rts			;-=> exit, OK
                
nnumber		ldx #commands3-commands
                gne channel15_cont.htnuo
		.pend
;-------------------------------
;S- swap device number
;
;"S-8" swap 8
;"S-9" swap 9
;"S-D" swap default
;
;It's not a real swap...
;
cmd_s		.proc
                cpy #"d"
                beq nj3
                tya
                eor #$38
                lsr
                bne p4_null_one.cmdnotimpx  	;not implemented
                rol
                ora #8
ej		gpl usercmd0.in

nj3		jsr OP_get_current_drive_type
                lda setup_drivenumbers_exp,x
                and #$0f
                clc
                adc #7
                gcc ej
                .pend
;-------------------------------
;U0>L0 Unlock disk
;U0>L1 Lock disk
;
cutil_l		.proc
                jsr cutil_e.cucc
                ldx #ATAPI_CMD_LOCK_UNLOCK
in		#pw 4

do_atapicommandx .proc
		txa
do_atapicommand	.proc
		sta packet_buff_exp+11
		jsr OP_atapi_command
		jmp LWOP_reporterror_xit;-=> exit, OK
                .pend
;-------------------------------
;U0>CS
;
cd_playstop   	.proc
		ldx #ATAPI_CMD_STOP_PLAY_SCAN
		.byte SKIP2
;-------------------------------
;U0>CP
;
cd_playpauseresume .proc		;"U0>CP"+CHR$(PAUSE/RESUME)
		ldx #ATAPI_PAUSE_RESUME
                lda path_exp+5
                and #1
                #pw 8
		gpl do_atapicommandx
                .pend
;-------------------------------
;"U0>CA"+CHR$(FEND)+CHR$(SEND)+CHR$(MEND)+CHR$(FST)+CHR$(SST)+CHR$(MST)
;
cd_playaudiomsf	.proc
                ldx #5
lu      	lda path_exp+5,x
                sta packet_buff_exp+3,x
                dex
                bpl lu
                lda #ATAPI_PLAY_AUDIO_MSF
                gne do_atapicommand
                .pend
;-------------------------------
;"U0>CF"+CHR$(ADR1)+CHR$(ADR2)+CHR$(ADR3)+CHR$(ADR4)+CHR$(MODE)
;
cd_fastfw   	.proc
                ldx #251
lu      	lda path_exp-246,x
		.if SUPERCPU=1
                sta (packet_buff_exp-245) & $1ffff,x
		.else
                sta (packet_buff_exp-245) & $ffff,x
                .fi
                inx
                bne lu
                tay
                and #$c0
                #pw 9
                tya
                and #$10
                #pw 1
                lda #ATAPI_SCAN_AUDIO
                gne do_atapicommand
                .pend
;-------------------------------
;U0>C cdrom commands
;
cutil_c         .proc
		jsr XIT_init_drive2
                jsr OP_atapi_packet
                lda path_exp+4
                ldx #commands4-commands
                gne channel15_cont.htnuo
;-------------------------------
;U0>Cv Sense volume settings
;
cd_volume2	.proc
		jsr cd_volume.mode_sense3

                #ldx_l_a ram_exp_buffer0_exp+1
                ldy #8
capin           sty pom1
sdj             lda ram_exp_buffer0_exp+1,x
                pha
                dex
                dey
                bne sdj
                lda pom1
                gne channel_ret
;-------------------------------
;U0>CC Get drive capabilities
;
cd_readcap     	.proc
		lda #$2a
		jsr cd_volume.mode_sense2
                ldx #$16
                ldy #14
                gne cd_volume2.capin
;-------------------------------
;U0>CV Set volume
;
cd_volume      	.proc
		jsr mode_sense3
                jsr copy_b0_hi
                lda #$4e
		jsr mode_sense2
                
                ldy #7
                #ldx_l_a ram_exp_buffer0_exp+$101
lp              lda ram_exp_buffer0_exp+1,x
                beq nc
                #and_l_y_ path_exp+5
                sta ram_exp_buffer0_exp+$101,x
nc		dex
		dey
                bpl lp

fh      	jsr mode_select_st
                jsr LWOP_reporterror_write_xit
                jsr mode_select_en
	        jmp LWOP_reporterror_write_xit;-=> exit, OK

mode_select_st 	lda #ATAPI_MODE_SELECT
                jsr OP_atapi_packet
                #pc 1,$10
                lda ram_exp_buffer0_exp+$101
                clc
                adc #2
                #pw 8
                jmp OP_atapi_dataout_command

mode_sense3	lda #$0e
mode_sense2   	jsr mode_sense
                jmp LWOP_reporterror_xit
                
mode_sense	pha
		lda #ATAPI_MODE_SENSE
		jsr OP_atapi_packet
                pla
                #pw 2
                #pc 8,$80
                jsr OP_atapi_datain_command
                bcs nj
                ldx #0
np              lda data_g
		#lda_low_reg
                sta ram_exp_buffer0_exp,x
                inx
                lda hig_reg
                sta ram_exp_buffer0_exp,x
                inx
                bpl np
                gmi nmdat
                
mode_select_en  lda ram_exp_buffer0_exp+$101
                lsr
                tay
                ldx #0
lue             lda ram_exp_buffer0_exp+$101,x
                sta hig_reg
                lda ram_exp_buffer0_exp+$100,x
                #sta_low_reg
                sta data_g
                inx
                inx
                dey
                bpl lue
                jsr wait_busy           ;wait until writing finished
		bcs nj
nmdat           jmp ata_pi_nomore_data

nj            	rts
		.pend
;-------------------------------
;Idle off in 16 secs (for setfeatures)
;
cdrom_idleoff   .proc
		lda #$0d
		jsr cd_volume.mode_sense
                bcs gon
                jsr copy_b0_hi

                lda #$4d
		jsr cd_volume.mode_sense
                bcs gon

                lda ram_exp_buffer0_exp+$00b
                beq nce
                and #8
                sta ram_exp_buffer0_exp+$10b
nce
		jsr cd_volume.mode_select_st
                bcc cd_volume.mode_select_en
gon             rts
		.pend
                
copy_b0_hi      .proc
		ldx #0
nu              lda ram_exp_buffer0_exp,x
                sta ram_exp_buffer0_exp+$100,x
                inx
                bpl nu
		rts
                .pend
;-------------------------------
;G get partition
;
;"G-P"           Current partition
;"G-P"+CHR$(0)   System partition?
;"G-P"+CHR$(255) Current partition
;"G-P"+CHR$(k)   Other partition
;
;1st byte: 0 not created, 1 native, 2 1541, 3 1571, 4 1581, 5 1581 cpm, 6 Spool, 7 Foreign, 255 system
;2nd byte: info
;3rd byte: partition
;4-19th byte: Partition name
;20-22th byte: Partition start
;23-27th byte: Unused
;28-30th byte: size of partition

cmd_g		.proc
                bne nkqx
                cpy #"p"
                bne nkqx
                lda #255
                pha
                pha
                #lda0
                ldx #9
aa2             pha
                dex   
                bne aa2
                lda #$20		;fake partition name
                ldx #15
aa1             pha
                dex
                bne aa1
		lda path_exp+3
                cmp #255
                bne *+4
                lda #0
                pha

		jsr XIT_init_drive	;-=> error

                pla
                jsr OP_check_partition_selection
                ldx #0
                bcs tt
                iny
                ldx #1                  ;Native partition
tt              #phy_a
		#lda0
                pha
                #phx_a

                lda #30
                gne channel_ret
		.pend

nkqx		jmp cmdnotimp
;-------------------------------
;U0>CM Get media type
;
cd_media       	.proc
		jsr cd_volume.mode_sense3
                lda ram_exp_buffer0_exp+2
                #jmps return1
;-------------------------------
;U0>P
;
cutil_P		.proc		;"u0>P"
		jsr select_drive
		ldx #check_power_mode1_d
		lda #check_power_mode2_d
                jsr try2
		lda nsector_g
                #lda_low_reg
                #jmps return1
		.pend
;-------------------------------
;Fake memory read and write
;
;fea0 "CMD HD" CMD HD
;fea0 "CMD RL" CMD RamLink
;fea0 "CMD RD" CMD RamDrive
;e5c6 "4"      1541
;e5c6 "7"      1571
;a6e9 "8"      1581
;fea0 "CMD FD" CMD FD
;
cmd_memory      .proc
                cpy #"w"
                beq channel_ret.rtss	;m-w
                cpy #"r"
                bne nkqx		;not implemented
                lda path_exp+5
                cmp #60
                bcc *+4
                lda #60
                tax
                dex
                clc
		adc path_exp+3
                tay

                lda setup_atribut2_exp	;CMD emulation?
                asl
                #stz_a pom1

lul             dey
		tya
		and #31
                bcs *+4
                ora #16
                tay
		lda ou,y
                pha
                inc pom1
                dex
                bpl lul

		lda pom1
                gpl channel_ret

ou              .text "cmd hd emulated "	;Fake drive ROM
ou2             .text "ide64 cartridge "	;Fake drive ROM
		.pend

return1         .proc
		pha
                lda #1
channel_ret     .proc
		tay
		.if SUPERCPU=1
		lda current_drive_exp
                tax
                tya
                .else
                ldx current_drive_exp
                .fi
                sta status_size_exp,x
                #lda0
                sta status_pos_exp,x
                .if DRIVES=4
                bit errledoff_g
                .fi
		txa
		lsr
		ror			;*64
		ror
		tax
sds             pla
                sta drive_status_exp,x
                inx
                dey
                bne sds
rtss		rts
                .pend
;-------------------------------
;T Time
;
;T-RA
;T-RB
;T-RD
;T-WA No modify! Use setup...
;T-WB
;T-WD
;
;SUN. 00/00/00 00:00:00 AM
;
cmd_t		.proc
                bne n
                cpy #"r"
                bne n

		jsr LWOP_read_clock

;00 -> 12 AM
;01 -> 01 AM
;..
;11 -> 11 AM
;12 -> 12 PM
;13 -> 01 PM
;..
;23 -> 11 PM
                lda hour_exp
                cmp #$12
                php
                tax
                bne *+4
                lda #$12
                cmp #$13
                bcc xq
                sei
                sed
                sbc #$12
xq              plp
		sta hour_exp
		#lda0
                bcc *+4
                lda #4
                sta ampm_exp

                lda path_exp+3
                eor #"a"
                beq asc
                cmp #"d"^"a"
                beq q
                cmp #"b"^"a"
n               bne cmdnotimp
		clc
q
                ldy #7
sds             ldx fk,y
                lda sec_exp,x
                bcc sk
                jsr bcd2bin	;C=1 on exit
sk              pha
                dey
                bpl sds
                pla
                #sbc1c
                pha
                lda #8
channel_retx    gne channel_ret

fk              .byte day_exp-sec_exp
		.byte year_exp-sec_exp
		.byte month_exp-sec_exp
		.byte date_exp-sec_exp
		.byte hour_exp-sec_exp
		.byte min_exp-sec_exp
		.byte sec_exp-sec_exp
                .byte ampm_exp-sec_exp
		
asc		lda #"m"
		pha
                lda #"a"
                bcc *+4
                lda #"p"
                pha
                lda #$20
                pha
                
                ldy #5
skd             ldx fk2,y
                lda sec_exp,x
                tax
                and #$0f
                ora #$30
                pha
                txa
                lsr
                lsr
                lsr
                lsr
                ora #$30
                pha
                lda dl,y
                pha
                dey
                bpl skd

		lda day_exp
		asl
		asl
                tay
                ldx #4
nuo             dey
		lda days,y
		pha
                dex
                bne nuo
                
                lda #25
                gne channel_retx
                
dl              .byte " // ::"
fk2		.byte month_exp-sec_exp
		.byte date_exp-sec_exp
                .byte year_exp-sec_exp
		.byte hour_exp-sec_exp
		.byte min_exp-sec_exp
		.byte sec_exp-sec_exp


days		.text "sun."
		.text "mon."
		.text "tues"
		.text "wed."
		.text "thur"
		.text "fri."
		.text "sat."
                .pend
;-------------------------------
;N format
;
cmd_n		.proc
                cmp #"="
                bne cmdnotimpx
                
                ldy #0
                clc
ne2             ldx #2
ne              lda formats,y
                bmi ni
                eor path_exp,x
                beq *+3
                sec
                iny
                inx
                gne ne

ni		bcs ee
		and #$7f
		eor path_exp,x
                beq joo
ee              iny
		iny
                iny
                iny
                cpy #formatsend-formats
                bcc ne2
cmdnotimpx      jmp cmdnotimp

joo		#phy_a
		jsr XIT_init_drive2
                
                jsr XIT_drive_writep
                
		lda #ATAPI_FORMAT_UNIT
		jsr OP_atapi_packet
		#pc 1,%00010111
		jsr OP_atapi_dataout_command

		#lda0
		jsr OP_atapi_packet

             	#pc 1,%10100010
		#pc 3,8
		#pc 10,>512
                
                #ply_a                
                ldx #2
lu              iny
		lda formats,y
                sta packet_buff_exp+4,x
                dex
                bpl lu

		ldx #11
lp		lda packet_buff_exp-1,x
		sta hig_reg
		lda packet_buff_exp,x	;Send the command to the device.
		#sta_low_reg
		sta data_g
                dex
		dex
		bpl lp
                jsr ata_pi_nomore_data
		bcs nj
		jsr wait_busy		;wait until writing finished
nj              jsr LWOP_reporterror_write_xit
		jmp invalidate_drive	;-=> exit, OK
                
formats         .text "720K"
                .byte $00,$05,$a0	;80/2/9 75Kbps 720rpm
                .text "1.2M"
                .byte $00,$09,$60	;80/2/15 125Kbps 720rpm
                .text "1.44M"
                .byte $00,$0b,$40	;80/2/18 150Kbps 720rpm
                .text "120M"
                .byte $03,$c3,$00	;963/8/32 533Kbps 720rpm
;                .text "100M"
;                .byte $00,$02,$ff,$e0	;96/64/32 4096Kbps 2941rpm (does not work)
;                .text "250M"
;                .byte $00,$xx,$xx,$xx	;239/64/32 (does not work)
formatsend
                .pend
;-------------------------------
;B block commands
;
;B=R and B=W and B=P and B=T and B=S
;
cmd_b		.proc
		cmp #"="
		bne cmd_n.cmdnotimpx
		tya
		ldx #commandslo-commands
                gne channel15_cont.htnuo
                .pend
;-------------------------------
;B=P block position
;
cbl_p		.proc
		jsr find_buffer_for_channel
		lda next_sector
		sec
		sbc #1
		sta buffer_address_low_exp,x
		php
		lda buffer_address_max_hi_exp,x
		clc
		sbc buffer_start,x
		plp
		and next_sector+1
		sbc #0
		clc
		adc buffer_start,x
		sta buffer_address_hi_exp,x
ok		rts				;-=> exit, OK
		.pend
;-------------------------------
;B=R block read
;
cbl_r		.proc
		jsr blockc
		pha
                jsr next_sector0
		beq ident
		pla
		jmp XIT_read_a_sector		;-=> exit, OK

ident		jsr OP_get_current_drive_type
		asl
		lda #ident_d
		bcc *+4
		lda #atapi_ident_d
		jsr ata_data_command
		jsr LWOP_reporterror_read_xit;-=> error
		pla
		jsr OP_read512_here
		jsr XIT_nomoreread		;-=> error
                jsr XIT_init_drive2
                bpl cbl_p.ok   			;not atapi
		lda #ATAPI_READ_FORMAT_CAPACITIES
		jsr OP_atapi_packet
                #pc 8,8
                jsr OP_atapi_datain_command
                bcc cue
                jsr OP_atapi_datain_command	;retry
                jsr LWOP_reporterror_xit	;-=> error
cue
		dec pom2
		dec pom2
                bit data_g
                bit data_g
		ldy #123
np              lda data_g
		#lda_low_reg
                #sta_zp_y pom1
                dey
                lda hig_reg
                #sta_zp_y pom1
                dey
                cpy #120
                bcs np
                gcc XIT_nomoreread		;-=> exit, OK
                .pend
;-------------------------------
;B=W block write
;
cbl_w		.proc
		jsr blockc
		pha
		lda direct_write_exp
		sec
		beq XIT_drive_writep.wp
		jsr XIT_drive_writep
		pla
		jmp XIT_write_a_sector		;-=> exit, OK
                .pend

XIT_drive_writep .proc
		.if SUPERCPU=1
                jsr OP_get_current_drive_type
                .else
		ldx current_drive_exp
                .fi
                #lda0
                cmp drive_allowwrite_exp,x
wp		jmp LWOP_reporterror_26_xit;write protected drive
                .pend
;-------------------------------
;B=S subchannel read
;
cbl_s		.proc
                jsr blockc
                pha
                lda #ATAPI_READ_SUB_CHANNEL
                jsr OP_atapi_packet
		lda next_sector+2
                #pw 3
                #ldatax next_sector
                and #$40
                #pw 2
                gpl cbl_t.re
                .pend
;-------------------------------
;B=T toc read
;
cbl_t		.proc
                jsr blockc
                pha
                lda #ATAPI_READ_TOC
                jsr OP_atapi_packet
		#ldatax next_sector
                and #$c0
                #pw 9
re              txa
                and #2
		#pw 1
                txa
                #andlsr $3c
                lsr
                #pw 2
	     	lda next_sector+1
                #pw 6
                #pc 7,>$800
                jsr OP_atapi_datain_command
                jsr LWOP_reporterror_read_xit;-=> error
                pla
                jsr OP_read2048_here
		jmp XIT_nomoreread	;-=> exit, OK
                .pend

;-----------------------
find_buffer_for_channel .proc
		#cnext path_exp+4 , next_sector
		lda path_exp+3
		jsr search_channel_A
		bne err2
		lda buffer_type_exp,x
		cmp #BUFFERMODE_DIRECT
		beq ok
err2		sec
		gcs LWOP_reporterror_70_xit

ok              rts
                .pend

blockc		.proc
		jsr find_buffer_for_channel
		jsr OP_init_buffer_pointers
		jsr XIT_init_drive2
                #jmps OP_buffer_start
		.pend
;-------------------------------
;I Initialize
;
; Syntax:
;
;"I" initialize current partition
;"I2" initialize second partition
;
; The current path is not affected!
;
cmd_i		.proc
		jsr XIT_init_drive	;Initialize
;-------------------------------
;V Validate
;
; Syntax:
;
;"V" Validate current partition
;"V2" Validate second partition
;
; Ignore, use cfsfsck!
;
cmd_v		.proc
                jsr OP_get_partnum	;I, V
                lda current_path
		jsr OP_check_partition_selection;Invalid? -=> error
                jmp LWOP_reporterror_74_xit
		.pend

commands	.text 0,"gusmntwbiv"
commands2	.text 0,"pPelc"
commands3	.text 0,"psavVmcf"
commands4	.text 0,"prwst"

		.if SUPERCPU=1
commandslo    	.word cmd_g
		.word cmd_user
		.word cmd_s
		.word cmd_memory
		.word cmd_n
		.word cmd_t
                .word cmd_w
                .word cmd_b
                .word cmd_i
                .word cmd_v
                .word commands
commandslo2     .word cutil_p
                .word cutil_P
                .word cutil_e
                .word cutil_l
                .word cutil_c
                .word commands                
commandslo3    	.word cd_playpauseresume
		.word cd_playstop
		.word cd_playaudiomsf
		.word cd_volume
		.word cd_volume2
		.word cd_media
                .word cd_readcap
                .word cd_fastfw
                .word commands                
commandslo4    	.word cbl_p
        	.word cbl_r
        	.word cbl_w
        	.word cbl_s
        	.word cbl_t
		.else                
commandslo    	.byte <(cmd_g-1)
		.byte <(cmd_user-1)
		.byte <(cmd_s-1)
		.byte <(cmd_memory-1)
		.byte <(cmd_n-1)
		.byte <(cmd_t-1)
                .byte <(cmd_w-1)
                .byte <(cmd_b-1)
                .byte <(cmd_i-1)
                .byte <(cmd_v-1)
                .byte <(commands-1)
commandslo2     .byte <(cutil_p-1)
                .byte <(cutil_P-1)
                .byte <(cutil_e-1)
                .byte <(cutil_l-1)
                .byte <(cutil_c-1)
                .byte <(commands-1)
commandslo3    	.byte <(cd_playpauseresume-1)
		.byte <(cd_playstop-1)
		.byte <(cd_playaudiomsf-1)
		.byte <(cd_volume-1)
		.byte <(cd_volume2-1)
		.byte <(cd_media-1)
                .byte <(cd_readcap-1)
                .byte <(cd_fastfw-1)
                .byte <(commands-1)
commandslo4    	.byte <(cbl_p-1)
		.byte <(cbl_r-1)
		.byte <(cbl_w-1)
		.byte <(cbl_s-1)
		.byte <(cbl_t-1)
commandshi     	.byte >(cmd_g-1)
		.byte >(cmd_user-1)
		.byte >(cmd_s-1)
		.byte >(cmd_memory-1)
		.byte >(cmd_n-1)
		.byte >(cmd_t-1)
                .byte >(cmd_w-1)
                .byte >(cmd_b-1)
                .byte >(cmd_i-1)
                .byte >(cmd_v-1)
                .byte >(commands-1)
commandshi2    	.byte >(cutil_p-1)
                .byte >(cutil_P-1)
                .byte >(cutil_e-1)
                .byte >(cutil_l-1)
                .byte >(cutil_c-1)
                .byte >(commands-1)
commandshi3    	.byte >(cd_playpauseresume-1)
		.byte >(cd_playstop-1)
		.byte >(cd_playaudiomsf-1)
		.byte >(cd_volume-1)
		.byte >(cd_volume2-1)
		.byte >(cd_media-1)
                .byte >(cd_readcap-1)
                .byte >(cd_fastfw-1)
                .byte >(commands-1)
commandshi4    	.byte >(cbl_p-1)
		.byte >(cbl_r-1)
		.byte >(cbl_w-1)
		.byte >(cbl_s-1)
		.byte >(cbl_t-1)
                .fi
