;
; chkout.asm - CHKOUT kernal extension handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;**************************************************
; Chkout (X=logical file number)
;
; Used registers: A X (Y preserved)
; Status: C
; C=1:
; A=3 file not open
; A=5 device not present
; A=7 (not output file)
; Status byte response:
; Cleared!
; ST=$03 timeout
; ST=$80 if device not present
; Secondary address 128..255 then communication on serial bus is a bit different...
;**************************************************
newchkout	.proc
		jsr search_logical_X_chkinout
		beq chkout_cont2	;Ok found buffer for file

		lda ActualDev
		sta buffer_drive_exp+MISCOUT
		#ldx_l_a open_files_currentp_exp
		lda open_files_info_exp,x
		ldx #MISCOUT
		sta buffer_info_exp+MISCOUT
		cmp #INFO_KERNAL
		bne chkout_cont3

		lda ActualDev
		cmp #3			;screen?
		bne *+4
		ldx #STDOUT
		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkout_doch
		.else
		#stx_l_ buffer_output_exp
		.fi

		cmp #4
		bcs serial      	;serial device?
		#psr $f25b
		gcc exit2		;keyboard, tape, rs232, screen

serial  	jsr chkinout_driveconv
                sta manager_chkout_exp
                
		jsr OP_cfg16up
		txa
                .if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr ieclisten
                .else
                
                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                php
                bcs nx
                jsr $ed0c		;listen
                jmp nx2
nx              jsr ieclisten
nx2		plp
                .else
                jsr $ed0c		;listen
                .fi

                .fi			;JIFFYDOS ACCEL CHRINOUT

		lda ActualSecAddr
		bpl sfc
		jsr $edbe		;clear atn
		#jmps xf
sfc
		.if JIFFYDOS_ACCEL_CHRINOUT=1
		jsr iecsecond
                .else
                
                .if ((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)
                bcs nx3
                jsr $edb9		;second
                jmp nx4
nx3             jsr iecsecond
nx4
                .else
                jsr $edb9		;second
                .fi
                
                .fi			;JIFFYDOS ACCEL CHRINOUT

xf		jsr OP_cfg16down
		txa
		bit Status
                bpl exit                ;to $f25d
                gmi exit_device_not_present

chkout_cont2	txa
		cmp buffer_output_exp	;execute command, if leaving channel
		beq chkout_cont3
		pha
		#ldatax buffer_output_exp
		sta buffer_currentp_exp	;OP_do_command_channel
		lda buffer_info_exp,x
		cmp #INFO_STATUS
		bne chkout_nstatus
                jsr chkoutclrchncmdchannel
chkout_nstatus	#plx_a

chkout_cont3	.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
		jsr chkout_doch
		.else
		#stx_l_a buffer_output_exp
		.fi

		lda ActualDev
exit		sta ActualOutput
		tax			;buggy software

		clc
exit2		jmp exit2basic
		.pend

		.if (CHRIN_CHROUT_RAM=1) & (SUPERCPU=0)
chkout_doch	.proc
		php
		pha
		txa
		sta buffer_output_exp
		clc
		adc #<buffer_address_low_exp
		sta newchrout_ram_at2_exp
		adc #buffer_address_hi_exp-buffer_address_low_exp
		sta newchrout_ram_at3_exp
		adc #buffer_info_exp-buffer_address_hi_exp
		sta newchrout_ram_at_exp
		pla
                plp
		rts
		.pend
		.fi
