;
; ideserv.asm - PClink handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;---------------------------------------
; Initiate communication (0 is returned)
;
;          1             1
;       __________________________
; C64  | ACCU | SECONDARY ADDRESS |
;      |______|___________________|
;
pclink_secaddr  = next_sector+3
pclink_timer	= current_path
pclink_crc	= current_path+2

pclink_init	.proc
                .if PCLINK_TYPE=3
		ldy #%11010000	;Resume both channels
		sty CRA
		pha
                pla
                .fi
                pha
                
                #stz_a pclink_timer+1
                .if PCLINK_USECRC=1
                #stz_aa pclink_crc
                .fi
		
		.if PCLINK_TYPE=1
		lda #%11000111
		and $dd00
		sta $dd00

lp		lda #63-8-32
		sta $dd02

		lda #63-32
		sta $dd02

                jsr pclink_timeout

                bit $dd00
                bvc lp

		lda #63
		sta $dd02
                pla
		.fi
		
		.if PCLINK_TYPE=2
		lda #$0f
		sta $dd03
		lda #63-4
		sta $dd02
		lda $dd00
		and #$fb
		sta $dd00
                pla
		.fi

		.if PCLINK_TYPE=3
		pla
                .if DUART_PORT=1
		ldy #%01010000	;set OP6=TXRDY_A, OP4=RDRDY_A
                .else
                ldy #%10100000	;set OP7=TXRDY_B, OP5=RDRDY_B
                .fi
		sty OPCR

		ldy #%10100000	;tr X=1 -- Set Tx BRG Select Extend Bit
		sty duart_command_register
		ldy #%10000000	;rx X=1 -- Set Rx BRG Select Extend Bit
		sty duart_command_register

;       	ldy #%10000000	;Bit Rate Set #2, Counter MODE-XTAL/16
		sty ACR

		ldy #DUART_SPEED*17  ;115.2 rx/115.2 tx
		sty duart_clock_select_register

		; Set mode registers
		ldy #%00010000	;Reset MR pointer
		sty duart_command_register

		ldy #%10010011	;RTS, Character mode, no parity, even, 8bit
		sty duart_mode_register
		ldy #%00010111	;Normal mode, CTS, 1 stop bit
		sty duart_mode_register

		ldy #%00100000	;Reset Receiver
		sty duart_command_register
		ldy #%00110000  ;Reset Transmitter
		sty duart_command_register
		ldy #%01000101	;reset ERROR status, enable Tx,Rx
		sty duart_command_register
                .if DUART_PORT=1
                ldy #%00000001
                .else
                ldy #%00000010
                .fi
                sty SOPBC	;activate RTS
		.fi

		jsr pclink_send
                #stz_a pclink_timer+1
		lda pclink_secaddr
;---------------------------------------
; Send a byte
;
;     C64 ____RDY RDY_____RDY RDY_____RDY RDY_____RDY RDY____
; ATN         \_____/     \_____/     \_____/     \_____/
;
;     PC  __GO   GO_____GO   GO_____GO   GO_____GO   GO_____
; DATA      \_____/     \_____/     \_____/     \_____/     
;               |     |     |     |     |     |     |     |
;     DATA____ _____ _____ _____ _____ _____ _____ _____ ___
; CLK     ____X_7___X_6___X_5___X_4___X_3___X_2___X_1___X_0_
;
pclink_send     .proc

		.if PCLINK_USECRC=1
                ldy pclink_crc
                eor crc8,y
                sta pclink_crc
                eor crc8,y
                .fi

		.if PCLINK_TYPE=1
		sec
                rol
		bcs cs1

cs4		ldy #63-8
v2		bit $dd00
		bmi v1
		sty $dd02
                
                asl
                bcc cs2

cs3		ldy #63-16
v4		bit $dd00
		bpl v3
		sty $dd02

		asl
                bcc cs4
                beq cs5

cs1		ldy #63-8-16
v6		bit $dd00
		bmi v5
		sty $dd02

		asl
		bcs cs3
                
cs2		ldy #63
v8		bit $dd00
		bpl v7
		sty $dd02

		asl
                bcc cs4
                bne cs1
cs5		rts

v1		jsr pclink_timeout
                gne v2

v3		jsr pclink_timeout
                gne v4

v5		jsr pclink_timeout
                gne v6

v7		jsr pclink_timeout
                gne v8
		.fi
		
		.if PCLINK_TYPE=2
		sta $dd01
		ldy #63
		sty $dd02
		lsr
		lsr
		lsr
		lsr
		tay
		lda #$10
v2		bit $dd0d
		beq v1
		sty $dd01
		ldy #63-4
		sty $dd02
v4		bit $dd0d
		beq v3
		rts

v1		jsr pclink_timeout
		gne v2

v3		jsr pclink_timeout
		gne v4
		.fi

		.if PCLINK_TYPE=3
		tay
                lda #%00000100		; txrdy?
sendloop        bit duart_status_register
		beq v1
		sty duart_tx_hold_register
		rts

v1		jsr pclink_timeout
		gne sendloop
		.fi

		.pend
;---------------------------------------
; Receive a byte (Z and N flags must be set correctly on exit, Y not touched!)
; C=1 on timeout and exit caller
;
;     C64 _______OK   OK_____OK   OK_____OK   OK_____OK   OK____
; ATN            \_____/     \_____/     \_____/     \_____/
;
;     PC  _____RDY RDY_____RDY RDY_____RDY RDY_____RDY RDY______
; DATA         \_____/     \_____/     \_____/     \_____/     
;               |     |     |     |     |     |     |     |
;     DATA___ _____ _____ _____ _____ _____ _____ _____ ________
; CLK     ___X_7___X_6___X_5___X_4___X_3___X_2___X_1___X_0______
;         
pclink_recieve	.proc

		.if PCLINK_TYPE=1
                lda #$fe
lp      	
		
v2		ldx $dd00
		bmi v1
                cpx #$40
                
                ldx #63-8
		stx $dd02

		rol

v4              ldx $dd00
                bpl v3
                cpx #$c0
                
                ldx #63
                stx $dd02

                rol
                bcs lp

		.if PCLINK_USECRC=1
docrc           ldx pclink_crc
		eor crc8,x
                sta pclink_crc
                eor crc8,x
                .fi
		rts

v1		jsr pclink_timeout
		gne v2

v3		jsr pclink_timeout
                gne v4
		.fi
		
		.if PCLINK_TYPE=2

		lda #63
		sta $dd02
		lda #$10
v2		bit $dd0d
		beq v1
		ldx $dd01
		lda #63-4
		sta $dd02
		lda #$10
v4		bit $dd0d
		beq v3
		txa
		lsr
		lsr
		lsr
		lsr
		ora $dd01
                .if PCLINK_USECRC=1
                ldx pclink_crc
                eor crc8,x
                sta pclink_crc
                eor crc8,x
                .fi
		rts

v1		jsr pclink_timeout
		gne v2

v3		jsr pclink_timeout
                gne v4
		.fi
                
		.if PCLINK_TYPE=3

receiveloop	lda duart_status_register
		lsr     	;rxrdy?
		bcc v1
                lsr
                lda duart_rx_hold_register
		bcs clear

                .if PCLINK_USECRC=1
re              ldx pclink_crc
                eor crc8,x
                sta pclink_crc
                eor crc8,x
                .fi

                rts
clear
                .if DUART_PORT=1
                ldx #%00000001
                .else
                ldx #%00000010
                .fi
                stx SOPBC	;activate RTS
		.if PCLINK_USECRC=1
		gne re
		.else
                tax
                rts
		.fi

v1              jsr pclink_timeout
		gne receiveloop
		.fi
                
		.pend
;---------------------------------------
; Load a file
;
;         1            1                2        1   1        2        X     1
;       ___________________________________________                              ___
; C64  | "R" | SECONDARY ADDRESS | BYTES LO/HI | 0 |                            | CLOSE
;      |_____|___________________|_____________|___|                            |___
;   ___                                             ____________________________
; OPEN |                                           | ? | BYTES LO/HI| DATA | ST |
;   ___|                                           |___|____________|______|____|
;
;          1            1                2         1    1   1        2        X      1    1
;       __________________________________________________                                    ___
; C64  | !"R" | SECONDARY ADDRESS | BYTES LO/HI | CRC | 0 |                                  | CLOSE
;      |______|___________________|_____________|_____|___|                                  |___
;   ___                                                    __________________________________
; OPEN |                                                  | ? | BYTES LO/HI| DATA | CRC | ST |
;   ___|                                                  |___|____________|______|_____|____|
;
; $00 - OK
; $40 - EOF
; $42 - Too much
; 
pclink_load	.proc
		#xit_init
                bcs pclink_load2
                
		lda #$60
                sta pclink_secaddr
		jsr pclink_send_open
		bne filenf     ;readonly?

                ldx #<2
                stx Status		;load error
                ldy #>2
                sec
                jsr pclink_start_read

                inc next_sector
                beq rovid
		jsr pclink_recieve
		tay
                inc next_sector
                beq rovid
                #stz_a Status		;Ok
		jsr pclink_recieve
		ldx ActualSecAddr	;sec addr!=0?
		bne jklj
		ldy $c3
		lda $c4
jklj		sty $ae
		sta $af
rovid
                .if PCLINK_USECRC=1
                ldy #1			;device not present on error
                .fi
		#jmps pclink_endofframe

filenf		ldy #$42		;file not found
		sty Status
                sec
                rts
                .pend

pclink_load2	.proc
                ldx #<$ffff
                .if SUPERCPU=1
                txy
                .else
                ldy #>$ffff
                .fi
;                sec
                jsr pclink_start_read

eiqj		inc next_sector
                beq vx
vx2		
		.if PCLINK_TYPE=3

receiveloop	lda duart_status_register
		lsr     	;rxrdy?
		bcc v1
;                lsr
                lda duart_rx_hold_register
;		bcs clear

                .if PCLINK_USECRC=1
re              ldx pclink_crc
                eor crc8,x
                sta pclink_crc
                eor crc8,x
                .fi

                .else
                jsr pclink_recieve
		.fi
		jsr start_zeropage
		iny
		bne eiqj
                sty pclink_timer+1
                jsr pclink_stop
                lda $af
                cmp #$cf
                beq outm
		inc $af
		bne eiqj
outm            ldy #0
                sec
                rts			;out of memory

vx              inc next_sector+1
                bne vx2
		geq pclink_endofframe

		.if PCLINK_TYPE=3

v1              jsr pclink_timeout
		gne receiveloop

;clear
;                .if DUART_PORT=1
;                ldx #%00000001
;                .else
;                ldx #%00000010
;                .fi
;                stx SOPBC	;activate RTS
;		gne re
                .fi

		.pend

peek_r		.proc
		.logical start_zeropage
		sta cfg_stnd		;3
at1		lda $ff00,x		;3
		.if SUPERCPU=1
                jsr switch_16k		;SCPU bug!
                .else
		sta cfg_open		;3
                .fi
		jmp pclink_send         ;3
		.here
		.pend

pclink_saveerr  .proc
		lda #$80
                sta Status
                rts
		.pend
                
;---------------------------------------
; Timeout handler
;
pclink_timeout	.proc
		dec pclink_timer
		bne vvx
		dec pclink_timer+1
		beq pclink_errexit
vvx             rts
		.pend
;---------------------------------------
; Stop handler
;
pclink_stop	.proc
		jsr check_stop
                bne pclink_errexit.mv

pclink_errexit 	.proc
		#xit_exit
		.if PCLINK_TYPE=1
		lda #63
		sta $dd02
		.fi
		
		.if PCLINK_TYPE=2
		lda #63-4
		sta $dd02
		.fi

		ldy #1			;break
                sec
mv		rts
		.pend
;---------------------------------------
; Data transfer start
;
;        1              1                2        1
;       ____________________________________________
; C64  | ACCU | SECONDARY ADDRESS | BYTES LO/HI | 0 |
;      |______|___________________|_____________|___|
;                                                    ___ 
;                                                   | ? |
;                                                   |___|
;
;         1              1                2         1    1
;       ___________________________________________________
; C64  | !ACCU | SECONDARY ADDRESS | BYTES LO/HI | CRC | 0 |
;      |_______|___________________|_____________|_____|___|
;                                                           ___
;                                                          | ? |
;                                                          |___|
pclink_dt_write	.proc
                .if PCLINK_USECRC=1
                lda #"W"
                .else
		lda #"w"
                .fi
		clc
pclink_dt	.proc
                sta pclink_timer+1
                #phy_a
                #phx_a
                bcs ajj
                #ldx_l_a buffer_currentp_exp
                lda buffer_pclink_secaddr,x
                sta pclink_secaddr
ajj             lda pclink_timer+1
                jsr pclink_init
                pla
                jsr pclink_send
                pla
                jsr pclink_send_a_crc_0_r
                bne pclink_errexit		;frame error
                rts
                .pend
;---------------------------------------
; Blockwrite
;
; $00 - OK
;
pclink_readwrite .proc
		#xit_init
                bcc pclink_read
                
                #ldx_l_a buffer_output_exp
		lda buffer_address_hi_exp,x
                sec
                sbc buffer_start,x
                bcc none
                tay
                lda buffer_address_low_exp,x
                tax
                inx
                bne *+3
                iny
                jsr pclink_chrinout.in
                bcs vs2

		jsr OP_buffer_start
                #sbc1c
		sta buffer_address_hi_exp,x
		lda #255
		sta buffer_address_low_exp,x

none        	
		ldx #11
iukq		lda peek_r,x
		sta start_zeropage,x
		dex
		bpl iukq
                lda pom9
                sta peek_r.at1+1
                lda pom10
                sta peek_r.at1+2
		
                #ldatax write_size
                ora write_size+1
                clc
                beq vs2			;nothing to send

		txa
                eor #$ff
                sta pom9
                lda write_size+1
                tay
                eor #$ff
                sta pom10
                jsr pclink_dt_write

                tax
                geq xd

vv              jsr start_zeropage
		inx
                bne xd
                stx pclink_timer+1
                inc peek_r.at1+2
xd              inc pom9
		bne vv
                inc pom10
                bne vv
                jsr pclink_send_crc_0_r
                cmp #1
                bcs vs2                 ;crc error?
                sta write_size
                sta write_size+1	;all sent
vs2		rts
		.pend
;---------------------------------------
; Open a file
;
;         1             1              X       1   1
;       _________________________________________
; C64  | "O" | SECONDARY ADDRESS | FILENAME  | 0 |
;      |_____|___________________|___________|___|
;                                                 ___
;                                                | ? |
;                                                |___|
;
;          1            1              X       1     1    1   1
;       ____________________________________________________
; C64  | !"O" | SECONDARY ADDRESS | FILENAME  | 0 | CRC | 0 |
;      |______|___________________|___________|___|_____|___|
;                                                            ___
;                                                           | ? |
;                                                           |___|
; $00 readonly
; $02 read+write
; $04 writeonly
; $06 error
;
pclink_send_open .proc
		jsr pclink_status_reset
		.if PCLINK_USECRC=1
                lda #"O"
                .else
		lda #"o"
                .fi
		jsr pclink_init
		ldx #0
exq		lda path_exp,x
		.if PCLINK_USECRC=1
		beq pclink_send_a_crc_0_r
                .else
                beq pclink_send_0_r
                .fi
		jsr pclink_send
		inx
		gne exq

pclink_send_a_crc_0_r .proc
		.if PCLINK_USECRC=1
                jsr pclink_send
pclink_send_crc_0_r .proc
		ldy pclink_crc
                lda crc8,y
                .fi

pclink_send_a_0_r .proc
		jsr pclink_send
                .if PCLINK_USECRC=0
pclink_send_crc_0_r .proc
                .fi
pclink_send_0_r .proc
                #lda0
                jsr pclink_send
		#jmps pclink_recieve
		.pend
;---------------------------------------
; Close a file
;
;         1             1          1
;       _____________________________
; C64  | "C" | SECONDARY ADDRESS | 0 |
;      |_____|___________________|___|
;                                     ___
;                                    | ? |
;                                    |___|
;
;          1             1           1    1
;       ____________________________________
; C64  | !"C" | SECONDARY ADDRESS | CRC | 0 |
;      |______|___________________|_____|___|
;                                            ___
;                                           | ? |
;                                           |___|
pclink_send_close
		jsr pclink_status_reset

		.proc
                .if PCLINK_USECRC=1
                lda #"C"
                .else
		lda #"c"
                .fi
		jsr pclink_init
                #jmps pclink_send_crc_0_r
		.pend
;---------------------------------------
; Open a file
;
pclink_openclose
		.proc
		#xit_init
                
                lda ActualSecAddr
                sta pclink_secaddr
                
                bcs pclink_send_close

		jsr pclink_send_open
                #andlsr 6
                tay
                lda modes,y
                rts
modes		.byte INFO_BLK_READ,INFO_STATUS,INFO_BLK_WRITE,INFO_IGNORE
		.pend
;---------------------------------------
; Fill chrin buffer
;
pclink_status	.proc
		#xit_init

                lda #<pclink_status_exp
                sta pom1
                lda #>pclink_status_exp
                sta pom2
                .if SUPERCPU=1
                lda #1
                sta pom3
                .fi
                
                ldx #<64
                ldy #>64
                jsr pclink_start_read_clc
                lda next_sector
                cmp #<($ffff-64)
                lda next_sector+1
                sbc #>($ffff-64)
                bcc pclink_chrinout.vs
                gcs pclink_chrinout.vv
;---------------------------------------
; Chrin
;
;         1            1           1   1   1   1        2        X     1
;       _____________________________________ 
; C64  | "R" | SECONDARY ADDRESS | 0 | 8 | 0 |
;      |_____|___________________|___|___|___|
;                                             ____________________________
;                                            | ? | BYTES LO/HI| DATA | ST |
;                                            |___|____________|______|____|
;
;          1            1           1   1    1    1   1        2        X      1    1
;       ____________________________________________ 
; C64  | !"R" | SECONDARY ADDRESS | 0 | 8 | CRC | 0 |
;      |______|___________________|___|___|_____|___|
;                                                    __________________________________
;                                                   | ? | BYTES LO/HI| DATA | CRC | ST |
;                                                   |___|____________|______|_____|____|
;
pclink_chrinout	.proc
		.if SUPERCPU=0
                txa
                .fi
		#xit_init
		.if SUPERCPU=0
                tax
                .fi
                
in              jsr pclink_status_reset

		bcs pclink_chrout

		jsr OP_buffer_start_pom1_y
                
                ldx #<2048
                ldy #>2048
                jsr pclink_start_read_clc

vv              inc next_sector
		beq vx
vx2		jsr pclink_recieve
                #sta_zp_y pom1
                iny
                bne vv
                sty pclink_timer+1
                inc pom2
                lda pom2
                and #7
                bne vv
                geq ss
vx		inc next_sector+1
		bne vx2
ss              
		#jmps pclink_endofframe

vs		rts
		.pend
;---------------------------------------
; Flush chrout buffer
;
;         1            1                2        1   1    X     1   1
;       ___________________________________________     __________ 
; C64  | "W" | SECONDARY ADDRESS | BYTES LO/HI | 0 |   | DATA | 0 |
;      |_____|___________________|_____________|___|   |______|___|
;                                                   ___            ___
;                                                  | ? |          | ? |
;                                                  |___|          |___|
;
;         1             1                2         1    1   1    X      1    1   1
;       __________________________________________________     ________________
; C64  | !"W" | SECONDARY ADDRESS | BYTES LO/HI | CRC | 0 |   | DATA | CRC | 0 |
;      |______|___________________|_____________|_____|___|   |______|_____|___|
;                                                          ___                  ___
;                                                         | ? |                | ? |
;                                                         |___|                |___|
;
pclink_chrout	.proc
		txa
                eor #$ff
                sta pom4
                tya
                eor #$ff
                sta pom5
                jsr pclink_dt_write

		jsr OP_buffer_start_pom1_y
                
                ldx pom4
                #jmps xd

vv              .if SUPERCPU=1
		lda [pom1]
                .else
		ldy #0
		lda (pom1),y
                .fi
                jsr pclink_send
                inc pom1
                bne *+4
                inc pom2
xd              inx
		bne vv
                stx pclink_timer+1
                inc pom5
                bne vv
                geq pclink_send_crc_0_r

vs		rts
		.pend
;---------------------------------------
; Status reset
;
pclink_status_reset .proc
                lda #$80
                sta status_pclink_pos_exp
		rts
                .pend
;---------------------------------------
; Start read
;
;        2
;  ____________
; | BYTES LO/HI|
; |____________|

pclink_start_read_clc .proc
		clc
pclink_start_read .proc
		.if PCLINK_USECRC=1
                lda #"R"
                .else
		lda #"r"
                .fi
		jsr pclink_dt
                ldy #0
                jsr pclink_recieve
                sta next_sector
                jsr pclink_recieve
                sta next_sector+1
                rts
		.pend
;---------------------------------------
; Blockread
;
pclink_read	.proc
                ldx read_size
                ldy read_size+1
                jsr pclink_start_read_clc
		#sta_cfg_16

vv              inc next_sector
		beq vx
vx2		jsr pclink_recieve
                sta (pom9),y		;must be sta (pom9),y
                iny
                bne vv
                sty pclink_timer+1
                dec read_size+1
                inc pom10
                gne vv
vx		inc next_sector+1
		bne vx2
                tya
                eor #255
                sec
                adc read_size
                sta read_size
                bcs ef
                dec read_size+1
ef
;---------------------------------------
; End of read frame
;  __________
; | CRC | ST |
; |_____|____|
;
; C=1 on CRC error
; Z=0 on frame error
; A=ST
;
pclink_endofframe .proc
		.if PCLINK_USECRC=1
		jsr pclink_recieve
                lda pclink_crc
		cmp #1
                php
		.fi
		jsr pclink_recieve
                .if PCLINK_USECRC=1
                plp
                tax
                .else
                clc
                .fi
vs		rts
		.pend
