;
; restore.asm - NMI handler
;
; Written by
;  Kajtar Zsolt <soci@c64.rulez.org>
;
;    This file is part of IDEDOS the IDE64 disk operating system
;    See README for copyright notice.
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;

restore		.proc
		cld
		.if DE32_EXISTS=1
		ora #$fc
		.fi
		sta nmi_oldconfig_exp
		#phx_a
		#phy_a

		ldy $dd0d		;Test if CIA#2 called this NMI
		bpl restore_no_cia
		lda #$7f
		sta $dd0d
		ldy $dd0d
into_nmi_io	#psr $fe64

		.if DE32_EXISTS=1
		lda nmi_oldconfig_exp
		lsr
		inc $318
		bcs switch_norm
		lsr
		bcc switch_16k
		gcs switch_8k
		.else
		lda nmi_oldconfig_exp
		cmp #$ff
		inc $318
		bcs switch_16k
		gcc switch_norm
		.fi
		.pend
;----------------------------
restore_no_cia	.proc
		lda $dc02
                pha
                ldy $dc03
                ldx #$ff
                stx $dc02
                .if SUPERCPU=1
                stz $dc03
                .else
		inx
                stx $dc03
                .fi
                lda $dc00
                ldx #$7f
                jsr keytest
                sty $dc03
                pla
                sta $dc02

		lda nmi_oldconfig_exp
		cmp #$ff
		.if DE32_EXISTS=1
		bcc no_setup_press	;no monitor and setup in cfg16!
		.else
		bcs no_setup_press	;no monitor and setup in cfg16!
		.fi
;----------------------------     C= / RESTORE => MONITOR
		cpx #%11011111		;test C=
		bne ddp
                .if SUPERCPU=0
		txa
		tsx
		inx			;y
		inx			;x
		inx			;a
		inx			;sr
		inx			;l
		inx			;h
		ldy $100,x
		tax
		cpy #>nmi_return
		beq ddp
                .fi
		lda zp_valid_exp
		bne ddp
		lda #BANK
		jmp monitor
;----------------------------
ddp		bit msgmode		;no setup if program running
		bpl no_setup_press
		cpx #%11111101		;test '<-'
		beq restore_STOP
;----------------------------    '<-' / RESTORE => SETUP
no_setup_press	txa			;check stop key
		bmi ht
		ldx #$bd
                jsr keytest
		inx
		bne ht2
ht		sta StopFlag
ht2

		lda StopFlag
		cmp #$7f		;STOP pressed?
		bne restore.into_nmi_io
;----------------------------
restore_STOP	.proc
		lda StopFlag
configback	pha
		sta cfg_open
		lda zp_valid_exp
		beq norestorezp
		jsr OP_restore_zeropage
norestorezp    	lda #$37
		sta $01
		inc $318		;Enable RUN/STOP+RESTORE to fix clrchn hang
		jsr clrchn_restore	;does cfg_16 too!

		jsr LWNO_set_new314vect

		jsr LWNO_init_io	;fda3

		.if SUPERCPU=1
                jsr scpu_default_opt
                .fi

		jsr LW_waitretrace     	;this eliminates flickering
					;when non standard colors
					;are used

		jsr LWNO_init_vic	;$e518

		#sta_cfg_open

		jsr LWOP_set_screen_col
		jsr LWOP_init_c128key

		#lda0			;Interrupt current HDD command (nop_d=0!!!)
                sta 198			;no keys in buffer
		sta kill_port		;Just to be sure...
		sta load_autostart_exp	;no autostart
                .if (((JIFFYDOS_ACCEL=1) & (SUPERCPU=0)) | (DOLPHINDOS_ACCEL=1)) & (JIFFYDOS_ACCEL_CHRINOUT=0)
                sta dos_accel_internal_exp
                .fi

		jsr check_drive_status

		pla
		cmp #$7f		;STOP pressed?
		beq tnjq
		jsr call_setup
		lda #$7f
		gne configback

tnjq		#psr $e37b		;# jmp ($a002)

		.if SUPERCPU=0
		lda JiffyDos_present_exp
		#sta_cfg_16
		beq js			;This is a bug in jiffydos...
		lda $300
		cmp #$39
		bne js
		lda $301
		cmp #$f7
		bne js
		lda $e447
		sta $300
js
                .fi

		inc $318		;Enable RUN/STOP+RESTORE
		jmp $e544		;clear screen and exit
		.pend

keytest         .proc
		stx $dc00

		ldx $dc01
		cpx $dc01
		bne *-6
                sta $dc00
                rts
                .pend
